#![feature(iter_intersperse)]
#![feature(trait_alias)]
use std::io::{BufRead, Write};
use command::{Command, Param};

mod command;

fn main() {
    let stdin = std::io::stdin();
    let mut in_lock = stdin.lock();

    let stdout = std::io::stdout();
    let mut out_lock = stdout.lock();

    let commands = &Command::all_commands();
    println!("Loaded {} commands", commands.len());

    loop {
        let mut buf = String::new();

        let ps1 = "> ".as_bytes();
        out_lock.write(ps1).unwrap();
        out_lock.flush().unwrap();

        in_lock
            .read_line(&mut buf)
            .expect("Failed to read in to buffer.");

        let mut args: Param = &buf.trim().split(' ').collect::<Vec<&str>>();
        let mut options = commands;

        loop {

            match args.get(0) {
                Some(cmd_arg) => {
                    // Try to find a subcommand to run
                    match Command::choose(cmd_arg, options) {
                        Some(choice) => {
                            if choice.sub_commands.is_empty() {
                                // No sub commands left to choose from, just pass the input
                                choice.call(&args);
                                break;
                            } else {
                                args = &args[1..];
                                options = &choice.sub_commands;

                                // a sort of last chance before running into hella edgecases
                                if args.is_empty() {
                                    choice.call(args);
                                    break;
                                }
                            }
                        }
                        // When no good choice presents it's self
                        None => break,
                    }
                }
                None => todo!("Couldn't get index 0 of the args"),
            }
        }
    }
}

